/****************************************************************************
*                                                                           *
*                            Third Year Project                             *
*                                                                           *
*                            An IBM PC Emulator                             *
*                          For Unix and X Windows                           *
*                                                                           *
*                             By David Hedley                               *
*                                                                           *
*                                                                           *
* This program is Copyrighted.  Consult the file COPYRIGHT for more details *
*                                                                           *
****************************************************************************/

/* This is MAIN.C  This is where everything begins... */


#include "global.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <memory.h>

#include "bios.h"
#include "vga.h"
#include "xstuff.h"
#include "hardware.h"
#include "vgahard.h"

BYTE	*memory;
char	*progname;
int	errors = 0;

void exit_emu(void)
{
	disable();
	bios_off();
	vga_off();
	exit(0);
}


void check_error(char *msg, int line)
{
	if (msg) {
		fprintf(stderr,"%s, line %d\n", msg, line);
		errors++;
	}
}


void read_pcemurc(void)
{         /* This procedure is a bit of a hack :) - will be redone later */
	FILE *f1;
	char buffer[1024];  /* Maximum path length. Should really be a #define */
	char keyword[1024];
	char value[1024];
	int line = 0;
	int tmp;

	/* Try current directory first... */

	if ((f1 = fopen(".pcemurc","r")) == NULL)
	{           /* Try home directory */
		sprintf(buffer,"%s/.pcemurc", getenv("HOME"));
		if ((f1 = fopen(buffer,"r")) == NULL)
		{
			/*printf("Warning: .pcemurc not found - using compile time defaults\n");*/
			return;
		}
	}

	while (fgets(buffer,sizeof buffer,f1) != NULL)
	{
		line++;
		tmp = sscanf(buffer," %s %s", keyword, value);

		if (tmp == 0 || keyword[0] == '#')
			continue;
		if (tmp != 2)
		{
			check_error("Syntax error in .pcemu file", line);
			continue;
		}

		if (strcasecmp(keyword,"bootdisk") == 0)
			check_error(set_boot_disk(value), line);
		else if (strcasecmp(keyword,"drivea") == 0)
			check_error(set_drive_file(0, value), line);
		else if (strcasecmp(keyword,"driveb") == 0)
			check_error(set_drive_file(1, value), line);
		else if (strcasecmp(keyword,"drivec") == 0)
			check_error(set_drive_file(128, value), line);
		else if (strcasecmp(keyword,"updatespeed") == 0)
			check_error(set_update_rate(strtol(value, NULL,10)),
				line);
		else if (strcasecmp(keyword,"cursorrate") == 0)
			check_error(set_cursor_rate(strtol(value, NULL,10)),
				line);
		else if (strcasecmp(keyword,"wait") == 0)
			check_error(set_delay(strtol(value, NULL,10)), line);
		else if (strcasecmp(keyword,"x11") == 0)
			check_error(use_x11(strtol(value, NULL,10)), line);
		else if (strcasecmp(keyword,"x11-font") == 0)
			check_error(set_x11_font(value), line);
		else
			check_error("Syntax error in .pcemu file", line);
	}
	fclose(f1);
}           


void usage(char *name) {
	fprintf(stderr, "Usage: %s\t"
		"[-b <boot drive ID>]\n"
		"\t\t[-c <cursor speed>]\n"
		"\t\t[-d A|B:<floppy image>:<type>\n"
		"\t\t[-d C:<harddisk image>:<cyl>:<head>:<sec>]\n"
		"\t\t[-f <font>]\n"
		"\t\t[-u <screen update rate>]\n"
		"\t\t[-w <delay>]\n"
		"\t\t[-x 0|1]\n",
		name);
	exit(1);
}

int main(int argc, char **argv)
{
	int	i, j;

	progname = (progname = strrchr(argv[0],'/')) ? progname+1 : argv[0];

#ifndef BOOT    
	FILE *f1;
	if (argc != 2)
	{
		fprintf(stderr,"Usage: %s testfile\n",progname);
		exit(1);
	}
#endif
	if (!(memory = (BYTE *)malloc(MEMORY_SIZE)))
	{
		fprintf(stderr,"Insufficient available memory\n");
		exit(1);
	}
#ifdef BOOT
	memset(memory,0, MEMORY_SIZE);
#else
	if (!(f1 = fopen(argv[1],"rb")))
	{
		fprintf(stderr,"Cannot open test file\n");
		exit(1);
	}


	fread(memory+0x800,1,MEMORY_SIZE-0x800,f1);
	fclose(f1);
#endif

	read_pcemurc();
	disable();

#ifdef BOOT
	for (i=1; i<argc; i++) {
		if (argv[i][0] != '-') usage(progname);
		for (j=1; argv[i][j]; j++) {
			switch (argv[i][j]) {
			case 'b':
				if (i >= argc-1) usage(progname);
				check_error(set_boot_disk(argv[++i]), 0);
				j = strlen(argv[i])-1;
				break;
			case 'c':
				if (i >= argc-1) usage(progname);
				check_error(set_cursor_rate(
						strtol(argv[++i], NULL,10)), 0);
				j = strlen(argv[i])-1;
				break;
			case 'd':
				if (i >= argc-1) usage(progname);
				check_error(set_drive_file(-1, argv[++i]), 0);
				j = strlen(argv[i])-1;
				break;
			case 'f':
				if (i >= argc-1) usage(progname);
				check_error(set_x11_font(argv[++i]), 0);
				j = strlen(argv[i])-1;
				break;
			case 'u':
				if (i >= argc-1) usage(progname);
				check_error(set_update_rate(
					strtol(argv[++i], NULL,10)), 0);
				j = strlen(argv[i])-1;
				break;
			case 'w':
				if (i >= argc-1) usage(progname);
				check_error(set_delay(strtol(argv[++i],
							NULL,10)), 0);
				j = strlen(argv[i])-1;
				break;
			case 'x':
				if (i >= argc-1) usage(progname);
				check_error(use_x11(
					strtol(argv[++i], NULL,10)), 0);
				j = strlen(argv[i])-1;
				break;
			default:
				usage(progname);
				break;
			}
		}
	}
#endif
	if (errors) exit(1);

	start_video();
	init_cpu();
	init_bios();
	init_vga();
	init_timer();

	enable();

	execute();
	
	/* NOT REACHED */
	return 0;
}
